/* timerb.lib sample application

	Generic TIMERB library use
	Support the following:
		1. periodic call to user routine every timer int
		2.	supports times from about 50us to 1500 seconds with 11.0592mHz xtal (22.1 clock)
		Interrupts faster than 20,000/sec start to become too much overhead for use
	TimerBinit() must be called to initialize the ISR
		uses BIOS freq_divider unless XTAL_FREQ is set to freq in hz
	Calls user-supplied function root nodebug void TimerBRoutine()
		if FAST_TIMERB is defined, only, hl,de,a,f are saved in ISR and TimerBRoutine
			must be asm code that save any regs it uses
Author: Scott Henion
Company: SHDesigns 				Web: http://www.shdesigns.org
*/

/* define this to use asm user function ISR. READ NOTES ON FAST_TIMERB
 asm version handles 40us rate on 22.1mhz board
 'c' version handles 50us
 anything lower than 100us, really slows everything down and
 if too fast, the watchdog timer resets board
 really should use asm if <250us */

// id this is defined link with TimerBFast.lib, otherwise use TimerBSlow.lib
#define FAST_TIMERB

// define this if you want to include multi-second delays.
#define TEST_LONG

#include <stdio.h>
#include "timerb.h"
#include "rabbit.h"
#include "dcdefs.h"

unsigned long counter;
unsigned long start_mstimer;
unsigned long last_mstimer;
long looptime;
int counterrun;

// calc ints/sec
void show_counter(void)
{
	if (counter)		// prevent divide by 0
		printf("counter=%lu T=%lu us\r\n",counter,(last_mstimer-start_mstimer)*1000l/counter);
	if (counter>2000000l)
	{
		counter=0;	// reset counter to prevent overflow
		start_mstimer=MS_TIMER;
	}
}


int init_timer(long ussec)
{
	float actual;
	if (ussec>4000000l)
		looptime=5000l;
	else
		looptime=1000l;
	printf("Timer init at %ld usec\n",ussec);
	counter=0;
	start_mstimer=MS_TIMER;
	if (TimerBInit(ussec,2))		// 10 ms, highest priority
	{
		counterrun=0;
		printf("Invalid value, timer not started.\n");
		return 1;
	}
	actual=TimerBRate();
	printf("Acual Timer ints are: %f usec err=%f%%\n",actual,
	100.0*(actual-(float)ussec)/(float)ussec);
	if (_timerb_scale)
		printf("\t Interrupt scaling=%u\n",_timerb_scale);
	else
		printf("\t Interrupt Scaling off\n");
	counterrun=1;
	return 0;
}

void DelayMs(unsigned long delay)
{
	unsigned long etime;
	etime=getMilliSeconds()+delay;
	while (getMilliSeconds()<etime);
}
/*

		user 'C' or asm routines go  here, make them quick!!
*/
#ifdef FAST_TIMERB
// this is fast asm version
// if we use anything other than HL,DE,AF: WE MUST SAVE IT
// we cant call any 'C' functions, unless we save EVERYTHING, but
// the idea hear is to be real fast and exit, so avoid 'C' overhead
// routine below uses and save bc (could have used DE with no save) but
// this is example
void near _nostep TimerBRoutine(void)
{
#asm
//_TimerBRoutine::
	ld	hl,(_counter)
	inc hl
	ld (_counter),hl
	bool	hl
	jr nz,no_countc
	ld	hl,(_counter+2)
	inc	hl
	ld (_counter+2),hl
no_countc:
	ld	hl,(__TimerMilliSeconds+4)
	ld (_last_mstimer),hl
	ld	hl,(__TimerMilliSeconds+6)
	ld (_last_mstimer+2),hl

	// toggle LED
	ld	a,2
	ld	hl,_srPEDR
	xor a,(hl)
	ld (hl),a
	ioi ld (PEDR),a
//	ret
#endasm
}
#else
// normal, slower 'c' function
// all regs are saved, try to be real small here if lots of ints.
void _near _nostep TimerBRoutine(void)
{
	counter++;
	last_mstimer=MS_TIMER;
	outi(PEDR,srPEDR^2);
}
#endif


void init_leds(void)
{
// copied form toggled.c
	/*  1. Convert the I/O ports.  Disable slave port which makes
	 *     Port A an output, and PORT E not have SCS signal.
	 */
	outi(SPCR, srSPCR=0x84);

	/*  2. Read function shadow and set PE1 and PE7 as normal I/O.
	 *     LED's are conencted to PE1 and PE7, make them outputs.
	 *     Using shadow register preserves function of other Port E bits.
	 */
	outi(PEFR,   srPEFR=  ~((1<<7)|(1<<1)) & srPEFR);
	outi(PEDDR,  srPEDDR=srPEDR|(1<<7)|(1<<1));

	/*  3. Turn on DS2 (1 -> PE1) and turn off DS3 (0 -> PE7).
	 */
	outi(PEDR, srPEDR=2);
}
const unsigned long int ustab[]={
									// test out of bounds vals for 11mhz xtal
									10l,1554000000l,
									1000l,  // first test is 1ms
#ifdef TEST_LONG
									5000000l,1000000l,500000l,250000l,100000l,
									50000l,25000l,10000l,5000l,2000l,
#endif
									500l,250l,125l,50l,
#ifdef FAST_TIMERB
									40l,
									25l,20l,
// will not work if debug (RST 28) enabled
									15l,
#endif
//									// test out of bounds vals for 11mhz xtal
//									10l,1554000000l,
									185000l // last is 185ms from msg board ;-)
									};
int reinit;

void near  _nostep TimerFunc(void)
{
	static int lastsw=0;
	// press s2 to go to next test
	if ((ini(PBDR)&0x4)==0)	// s1 pressed
	{
		if (!lastsw)
			lastsw=reinit=1;
	}
	else
		lastsw=0;
	outi(PEDR, srPEDR^=0x80);
}

extern unsigned char _serial19200TC;

void main(void)
{
	int i,loops=10;
	reinit=0;
	counterrun=0;
	i=0;
    WDT_DISABLE();
	init_leds();
	printf("_serial19200TC=%d",_serial19200TC);
	printf("Free xmem: %ld\n",xavail());
	printf("SEGSIZE=%x\n",ini(SEGSIZE));
    //outi(TAT1R,srTAT1R=15);
//	startTimer(100,0,1);
	startTimer(8,TimerFunc,1);
    ipset0();
	init_timer(ustab[i]);	// run first test
	while (1)
	{
		if (reinit)
		{

			TimerBUninit();		// important to un-init before restarting
			if (++i==(sizeof(ustab)/sizeof(unsigned long)))
				i=0;	// redo test
			reinit=init_timer(ustab[i]);	// run current test
			loops=10;
		}
		DelayMs(looptime);
		show_counter();			// 1st call init ctr
		if (--loops<=0)
			reinit=1;
	}
}


